function varargout = current2TikZ(data,settings)
%% low-level conversion function for plotCurrent()
% internal package function
% input structures described in documentation
%
%  2020, Vojtech Neuman, neumavo1@fel.cvut.cz (2020)

%% Data Acquisition
cVec = data.cVec;
cMap = data.cMap;

vert = data.vert*settings.TransMat3D;
vert_m = mean(vert);
vert = vert-vert_m;

arrowB = data.arrows(:,1:3)*settings.TransMat3D-vert_m;
arrowE = data.arrows(:,4:6)*settings.TransMat3D-vert_m;
arrowV = data.arrowsVert;

faces = data.faces_b;
faces_n = size(faces,1);
opacity = settings.TikZ_Opacity;

%% Conversion Modification
% area correction factor
faceArea = 0.5*sqrt(sum((cross(vert(faces(:,1),:)-vert(faces(:,3),:),...
   vert(faces(:,2),:)-vert(faces(:,3),:))).^2,2));
faceAreaCorr = sum(faceArea);

% opacity settings
if isscalar(opacity)
   if opacity ~= 1
      settings.Compression = false;
   end
   opacity = ones(faces_n,1).*[opacity opacity];
elseif ~isvector(opacity) || ~(length(opacity) == faces_n)
   error('Errow with opacity settings.')
else
   opacity = [1 1].*opacity;
   settings.Compression = false;
end
opacity(opacity(:,1) < 0.09,1) = 0.09;

% sorting
cvert = zeros(faces_n,3);
for j = 1:faces_n
   cvert(j,:) = mean(vert(faces(j,:),:),1);
end
[~,ind] = sort(cvert(:,3));
faces = faces(ind,:);
cVec = cVec(ind,:);
arrowB = arrowB(ind,:);
arrowE = arrowE(ind,:);
opacity = opacity(ind,:);

% compression process
if settings.Compression
   cvert = cvert(ind,:);
   faces_i = export.advanced.utilities.reduceTriangle(vert,faces,cvert);
   faces = faces(faces_i,:);
   cVec = cVec(faces_i,:);
   arrowB = arrowB(faces_i,:);
   arrowE = arrowE(faces_i,:);
   opacity = opacity(faces_i,:);
   faces_n = size(faces,1);
end

% reordering
faces = faces(end:-1:1,:);
cVec = cVec(end:-1:1);
arrowB = arrowB(end:-1:1,:);
arrowE = arrowE(end:-1:1,:);
opacity = opacity(end:-1:1,:);
vert = vert*[1 0 0; 0 -1 0; 0 0 1];
arrowB = arrowB*[1 0 0; 0 -1 0; 0 0 1];
arrowE = arrowE*[1 0 0; 0 -1 0; 0 0 1];

% circum sphere radius calculation
distMatrix =...
   models.utilities.geomPublic.euclideanDistanceBetweenTwoSetsSqrt(vert,vert);
diameterOfCircumSphere = max(max(distMatrix));
[ai, bi] = find(distMatrix == diameterOfCircumSphere);
centerOfCircumSphere = mean(vert([ai(2),bi(2)],:));
radiusOfCircumSphere = diameterOfCircumSphere / 2;
vert = vert - centerOfCircumSphere;
arrowB = arrowB - centerOfCircumSphere;
arrowE = arrowE - centerOfCircumSphere;

%% Strings Preparation
% LaTeX environment settings
if settings.Standalone
   introStr = sprintf([...
      '\\documentclass[tikz]{standalone}\n'...
      '\\usepackage{amsmath}\n'...
      '\\usepackage{tikz}\n'...
      '\\usetikzlibrary{arrows.meta}\n\n'...
      '%% Converted by AToM2TikZ\n',...
      '\\begin{document}\n']);
   endStr = sprintf([...
      '\\end{tikzpicture}\n'...
      '\\end{document}']);
else
   introStr = sprintf('%% Converted by AToM2TikZ\n');
   endStr = sprintf('\\end{tikzpicture}');
end
beginStr = sprintf('\\begin{tikzpicture}[scale=%1.2f]\n',...
   settings.TikZ_Scale);

% style strings
faceStr = sprintf(['\\tikzstyle{faceStr} = '...
   '[draw=black,line width=%1.2f,rounded corners=%1.2f,cap=round,fill]\n'],...
   settings.TikZ_Scale/20,settings.TikZ_Corners);
arrStr = sprintf(['\\tikzstyle{arrStr} = '...
   '[line width=%1.2f,color=black,cap=round]\n'],...
   settings.TikZ_Scale/20*exp(-1/2));

%% Strings Printing
fid = fopen([settings.FileName '.tex'], 'w+');
fprintf(fid,'%s',introStr);
fprintf(fid,'%s',beginStr);
fprintf(fid,'%% ===== Line Styles =====\n');
fprintf(fid,'%s',faceStr);
fprintf(fid,'%s\n',arrStr);

if settings.CircumSphere
   export.advanced.utilities.drawCircumSphere(fid,...
      radiusOfCircumSphere,settings);
end

if settings.Colorbar
   export.advanced.utilities.drawColorbar(fid,cMap,...
      radiusOfCircumSphere,settings);
end

% correction factor calculation
arrowArea = 0.5*sqrt(sum((cross(arrowV(faces(:,1),:)-arrowV(faces(:,3),:),...
   arrowV(faces(:,2),:)-arrowV(faces(:,3),:))).^2,2));
arrowSize = settings.TikZ_Scale/(11.5 + (settings.TikZ_Scale-1)*0.75)*...
   nthroot(arrowArea/max(arrowArea),2)*(mean(faceArea)/7.9365e-4)^(1/2);

% arrow threshold settings
if isscalar(uniquetol(arrowArea,1e-9))
   arrowSize = arrowSize / sqrt(2);
   threshold = 0;
else
   idxTemp = floor(length(arrowSize)*settings.ArrowThresh);
   if idxTemp == 0
      threshold = 0;
   else
      threshold = sort(arrowSize);
      threshold = threshold(idxTemp);
   end
end

fprintf(fid,'%% ===== Converted Plot =====\n');
for j = 1:faces_n
   tria = sprintf([...
      '\\draw['...
      'color={rgb,255:red,%3.2f;green,%3.2f;blue,%3.2f},'...
      'faceStr,'...
      'draw opacity=%1.4f,'...
      'fill opacity=%1.4f]'...] '...
      '(%1.4f,%1.4f) -- '...
      '(%1.4f,%1.4f) -- '...
      '(%1.4f,%1.4f) -- cycle;\n'],...
      255*cMap(cVec(j),1),...
      255*cMap(cVec(j),2),...
      255*cMap(cVec(j),3),...
      opacity(j,1),...
      opacity(j,2),...
      vert(faces(j,1),1),...
      vert(faces(j,1),2),...
      vert(faces(j,2),1),...
      vert(faces(j,2),2),...
      vert(faces(j,3),1),...
      vert(faces(j,3),2));
   fprintf(fid,'%s', tria);
   
   if arrowSize(j) > threshold
      arrw = sprintf([...
         '\\draw[-{Stealth[scale=%1.3f]},'...
         'arrStr,'...
         'draw opacity=%1.4f] '...
         '(%1.3f, %1.3f) --'...
         '(%1.3f, %1.3f);\n'],...
         arrowSize(j),...
         opacity(j,1),...
         arrowB(j,1),arrowB(j,2),...
         arrowE(j,1),arrowE(j,2));
      fprintf(fid,'%s', arrw);
   end
end

if settings.BoundingSphere
   export.advanced.utilities.drawBoundingSphere(fid,...
      radiusOfCircumSphere);
end

if settings.CoordinatesCross
   export.advanced.utilities.drawCoordinatesCross(fid,...
      faceAreaCorr,radiusOfCircumSphere,settings);
end

fprintf(fid,'%s',endStr);
fclose(fid);

% additional procedures
if settings.CompileAutomatically && settings.Standalone
   export.advanced.utilities.fileCompile(settings);
end

% output arguments
if nargout > 0
   varargout{1} = radiusOfCircumSphere;
end
end