function varargout = farField2TikZ(data,settings)
%% low-level conversion function for plotFarField()
% internal package function
% input structures described in documentation
%
%  2020, Vojtech Neuman, neumavo1@fel.cvut.cz (2020)

%% Data Acquisition
cVec = data.cVec;
cMap = data.cMap;

vert = data.vert*settings.TransMat3D;
vert_m = mean(vert);
vert = vert - vert_m;

faces = data.faces_b;
faces_n = size(faces,1);
opacity = settings.TikZ_Opacity;

%% Conversion Modification
% opacity settings
if isscalar(opacity)
   if opacity ~= 1
      settings.Compression = false;
   end
   opacity = ones(faces_n,1).*opacity;
elseif ~isvector(opacity) || ~(length(opacity) == faces_n)
   error('Errow with opacity settings.')
end
opacity(opacity(:,1) < 0.09,1) = 0.09;

% sorting
cvert = zeros(faces_n,3);
for j = 1:faces_n
   cvert(j,:) = mean(vert(faces(j,:),:),1);
end
[~,ind] = sort(cvert(:,3));
faces = faces(ind,:);
cVec = cVec(ind,:);
opacity = opacity(ind,:);

% area correction factor
faceA = faces(:,[1 2 3])';
faceB = faces(:,[1 3 4])';
facesTria = reshape([faceA;faceB],[3 faces_n*2])';
faceArea = 0.5*sqrt(sum((cross(vert(facesTria(:,1),:)-vert(facesTria(:,3),:),...
   vert(facesTria(:,2),:)-vert(facesTria(:,3),:))).^2,2));
faceAreaCorr = sum(faceArea);

% compression process
if settings.Compression && settings.TikZ_Opacity
   [faces_i,~] = export.advanced.utilities.reduceSquare(vert,...
      reshape([faceA;faceB],[3 faces_n*2])');
   faces = faces(faces_i(:,1) | faces_i(:,2),:);
   cVec = cVec(faces_i(:,1) | faces_i(:,2),:);
   opacity = opacity(faces_i(:,1) | faces_i(:,2),:);
   faces_n = size(faces,1);
end

% reordering
faces = faces(end:-1:1,:);
cVec = cVec(end:-1:1,:);
opacity = opacity(end:-1:1,:);
vert = vert*[1 0 0; 0 -1 0; 0 0 1];

% circum sphere radius calculation
distMatrix =...
   models.utilities.geomPublic.euclideanDistanceBetweenTwoSetsSqrt(...
   vert, vert);
diameterOfCircumSphere = max(max(distMatrix));
[ai, bi] = find(distMatrix == diameterOfCircumSphere);
centerOfCircumSphere = mean(vert([ai(2), bi(2)], :));
radiusOfCircumSphere = diameterOfCircumSphere / 2;
vert = vert - centerOfCircumSphere;

%% Strings Preparation
% Selection of primitives
if settings.Compression
   picData = [255*cMap(cVec,:)...
      255*cMap(cVec,:)...
      settings.TikZ_Scale/20.*ones(faces_n,1)...
      settings.TikZ_Corners.*ones(faces_n,1)...
      vert(faces(:,1),1:2)...
      vert(faces(:,2),1:2)...
      vert(faces(:,3),1:2)...
      vert(faces(:,4),1:2)]';
   
   picStr = sprintf([...
      '\\draw[fill,'...
      'color={rgb,255:red,%3.4f;'...
      'green,%3.4f;'...'
      'blue,%3.4f},'...
      'draw={rgb,255:red,%3.4f;'...
      'green,%3.4f;'...
      'blue,%3.4f},'...
      'line width=%1.3f,'...
      'rounded corners=%1.3f,'...
      'draw opacity=1,'...
      'fill opacity=1]'...
      '(%1.8f,%1.8f) -- '...
      '(%1.8f,%1.8f) -- '...
      '(%1.8f,%1.8f) -- '...
      '(%1.8f,%1.8f) -- cycle;\n'],picData);
else
   picData = [255*cMap(cVec,:)...
      opacity...
      vert(faces(:,1),1:2)...
      vert(faces(:,2),1:2)...
      vert(faces(:,3),1:2)...
      vert(faces(:,4),1:2)]';
   
   picStr = sprintf([...
      '\\fill[fill,'...
      'color={rgb,255:red,%3.4f;'...
      'green,%3.4f;'...
      'blue,%3.4f},'...
      'fill opacity=%1.3f]'...
      '(%1.8f,%1.8f) -- '...
      '(%1.8f,%1.8f) -- '...
      '(%1.8f,%1.8f) -- '...
      '(%1.8f,%1.8f) -- cycle;\n'],picData);
end

% LaTeX environment settings
if settings.Standalone
   introStr = sprintf([...
      '\\documentclass[tikz]{standalone}\n'...
      '\\usepackage{amsmath}\n'...
      '\\usepackage{tikz}\n'...
      '\\usetikzlibrary{arrows.meta}\n\n'...
      '%% Converted by AToM2TikZ\n',...
      '\\begin{document}\n']);
   endStr = sprintf([...
      '\\end{tikzpicture}\n'...
      '\\end{document}']);
else
   introStr = sprintf('%% Converted by AToM2TikZ\n');
   endStr = sprintf('\\end{tikzpicture}');
end
beginStr = sprintf('\\begin{tikzpicture}[scale=%1.2f]\n',...
   settings.TikZ_Scale);

%% Strings Printing
fid = fopen([settings.FileName '.tex'], 'w+');
fprintf(fid,'%s',introStr);
fprintf(fid,'%s',beginStr);

if settings.CircumSphere
   export.advanced.utilities.drawCircumSphere(fid,...
      radiusOfCircumSphere,settings);
end

if settings.Colorbar
   export.advanced.utilities.drawColorbar(fid,cMap,...
      radiusOfCircumSphere,settings);
end

fprintf(fid,'%% ===== Converted Plot =====\n');
fprintf(fid,'%s\n\n', picStr);

if settings.BoundingSphere
   export.advanced.utilities.drawBoundingSphere(fid,...
      radiusOfCircumSphere);
end

if settings.CoordinatesCross
   export.advanced.utilities.drawCoordinatesCross(fid,...
      faceAreaCorr,radiusOfCircumSphere,settings);
end

fprintf(fid,'%s',endStr);
fclose(fid);

% additional procedures
if settings.CompileAutomatically && settings.Standalone
   export.advanced.utilities.fileCompile(settings);
end

% output arguments
if nargout > 0
   varargout{1} = radiusOfCircumSphere;
end
end