function varargout = topoSens2TikZ(data,settings)
%% low-level conversion function for topological sensitivity results
% internal package function
% input structures described in documentation
%
%  2020, Vojtech Neuman, neumavo1@fel.cvut.cz (2020)

%% Data Acquisition
Mesh = data.Mesh;
BF = data.BF;
temp = data.Temp;
faces = Mesh.connectivityList;
vert = Mesh.nodes;
cTriaVert = Mesh.triangleCentroids;
cMap = [166 206 227; 255 255 153];

% rotation and translation transformations
cTriaVert = cTriaVert*settings.TransMat3D;
cTriaVert = cTriaVert - mean(cTriaVert);
cTriaVert = cTriaVert*[1 0 0; 0 -1 0; 0 0 1];

vert = vert*settings.TransMat3D;
vert = vert - mean(vert);
vert = vert*[1 0 0; 0 -1 0; 0 0 1];

%% Conversion Modification
distMatrix =...
   models.utilities.geomPublic.euclideanDistanceBetweenTwoSetsSqrt(...
   vert, vert);
diameterOfCircumSphere = max(max(distMatrix));
[ai, bi] = find(distMatrix == diameterOfCircumSphere);
centerOfCircumSphere = mean(vert([ai(2), bi(2)], :));
radiusOfCircumSphere = diameterOfCircumSphere / 2;
vert = vert - centerOfCircumSphere;
cTriaVert = cTriaVert - centerOfCircumSphere;

% data preparation
cVecTri = ones(Mesh.nTriangles,1);
cVecTri(unique(BF.data(temp,[2 4]))) = 2;
cVecEdge = ones(BF.nUnknowns,1);
cVecEdge(temp) = 2;
boundLines = setxor((1:Mesh.nEdges)', BF.data(:,3));
innerLines = BF.data(:,3);

%% Strings Preparation
if settings.Standalone
   introStr = sprintf([...
      '\\documentclass[tikz]{standalone}\n'...
      '\\usepackage{amsmath}\n'...
      '\\usepackage{tikz}\n'...
      '\\usetikzlibrary{arrows.meta}\n\n'...
      '%% Converted by AToM2TikZ\n',...
      '\\begin{document}\n']);
   endStr = sprintf([...
      '\\end{tikzpicture}\n'...
      '\\end{document}']);
else
   introStr = sprintf('%% Converted by AToM2TikZ\n');
   endStr = sprintf('\\end{tikzpicture}');
end

beginStr = sprintf('\\begin{tikzpicture}[scale=%1.2f]\n\n',...
   settings.TikZ_Scale);

% common styles
faceAreaStr1 = sprintf(['\\tikzstyle{triaFace1} = [color={rgb,255:red,%3d;'...
   'green,%d;blue,%d}]\n'],[255 255 255]);
faceAreaStr2 = sprintf(['\\tikzstyle{triaFace2} = [color={rgb,255:red,%3d;'...
   'green,%d;blue,%d}]\n'],cMap(2,:));
edgeStrB = sprintf(['\\tikzstyle{edgeStrB} = [color=black,'...
   'line width = 0.5,cap=round]\n']);
edgeStr1 = sprintf(['\\tikzstyle{edgeStr1} = [color=white!40!black,'...
   'line width = 0.5,cap=round]\n']);
edgeStr2 = sprintf(['\\tikzstyle{edgeStr2} = [color=white!80!black,'...
   'line width = 0.25,cap=round]\n']);
baseLineStr1 = sprintf(['\\tikzstyle{baseLineStr1} = [color=red!30!white,'...
   'line width = 0.25,cap=round]\n']);
baseLineStr2 = sprintf(['\\tikzstyle{baseLineStr2} = [color=red!70!black,'...
   'line width = 1,cap=round]\n']);

% faces
picData = [cVecTri...
   vert(faces(:,1),1:2)...
   vert(faces(:,2),1:2)...
   vert(faces(:,3),1:2)]';
faceStr = sprintf([...
   '\\fill[triaFace%d]'...
   '(%1.4f,%1.4f) -- '...
   '(%1.4f,%1.4f) -- '...
   '(%1.4f,%1.4f) -- cycle;\n'],picData);

% triangles edges
picData = [cVecEdge...
   vert(Mesh.edges(innerLines,1),1:2)...
   vert(Mesh.edges(innerLines,2),1:2)]';
innerStr = sprintf([...
   '\\draw[edgeStr%d]'...
   '(%1.4f,%1.4f) -- '...
   '(%1.4f,%1.4f);\n'],picData);
picData = [vert(Mesh.edges(boundLines,1),1:2)...
   vert(Mesh.edges(boundLines,2),1:2)]';
boundStr = sprintf([...
   '\\draw[edgeStrB]'...
   '(%1.4f,%1.4f) -- '...
   '(%1.4f,%1.4f);\n'],picData);

% disabled basis functions
picData = [cTriaVert(BF.data(~temp,2),1:2)...
   cTriaVert(BF.data(~temp,4),1:2)]';
if ~isempty(picData)
   baseStr1 = sprintf([...
      '\\draw[baseLineStr1]'...
      '(%1.4f,%1.4f) -- '...
      '(%1.4f,%1.4f);\n'],picData);
else
   baseStr1 = [];
end

% enabled basis functions
picData = [cTriaVert(BF.data(temp,2),1:2)...
   cTriaVert(BF.data(temp,4),1:2)]';
if ~isempty(picData)
   baseStr2 = sprintf([...
      '\\draw[baseLineStr2]'...
      '(%1.4f,%1.4f) -- '...
      '(%1.4f,%1.4f);\n'],picData);
else
   baseStr2 = [];
end

%% Strings Printing
fid = fopen([settings.FileName '.tex'], 'w+');
fprintf(fid,'%s',introStr);
fprintf(fid,'%s',beginStr);
fprintf(fid,'%% ===== Faces styles =====\n');
fprintf(fid,'%s',faceAreaStr1);
fprintf(fid,'%s',faceAreaStr2);
fprintf(fid,'%% ===== Line styles =====\n');
fprintf(fid,'%s',edgeStr1);
fprintf(fid,'%s',edgeStr2);
fprintf(fid,'%s',edgeStrB);
fprintf(fid,'%s',baseLineStr1);
fprintf(fid,'%s\n',baseLineStr2);

if settings.CircumSphere
   export.advanced.utilities.drawCircumSphere(fid,...
      radiusOfCircumSphere,settings);
end

fprintf(fid,'%% ===== Faces =====\n');
fprintf(fid,'%s\n\n',faceStr);
fprintf(fid,'%% ===== Edges =====\n');
fprintf(fid,'%s',innerStr);
fprintf(fid,'%s\n\n',boundStr);
fprintf(fid,'%% ===== Basis functions =====\n');
if ~isempty(baseStr1)
   fprintf(fid,'%s',baseStr1);
end
if ~isempty(baseStr2)
   fprintf(fid,'%s',baseStr2);
end
fprintf(fid,'%s', endStr);
fclose(fid);

% additional procedures
if settings.CompileAutomatically && settings.Standalone
   export.advanced.utilities.fileCompile(settings);
end

% output arguments
if nargout > 0
   varargout{1} = radiusOfCircumSphere;
end
end