function animatePlot2TikZ(hndl,azimuth,elevation,varargin)
%% animatePlot2TikZ() prepares .tex files for animated plot
% Converts AToM plots into TikZ macros.
%
%  INPUTS:
%   hndl                         structure with particular plot handles
%
%  INPUTS (optional):
%   'Data'                       data structure containing main plot
%                                information
%   'Settings'                   settings structure containing
%                                all or partial parameters
%   'Azimuth'                    specified azimuthal angle (have to be
%                                set together with 'elevation'),
%                                double [1 x 1] or [1 x N]
%   'Elevation'                  specified elevational angle (have to be
%                                set together with 'azimuth'),
%                                double [1 x 1] or [1 x N]
%   'Correction'                 factor for possible size correction,
%                                
%  Settable parts of Settings structure
%   'ArrowThresh'                sets threshold for omitting arrows,
%                                double [1 x 1]
%   'BoundingSphere'             draws gray circumscribing sphere around
%                                object, boolean [1 x 1]
%   'CircumSphere'               draws white sphere around whole picture,
%                                prepares data for animation, boolean [1 x 1]
%   'Colorbar'                   turn on/off colorbar with colormap
%                                boolean [1 x 1]
%   'CompileAutomatically'       turn on/off automatic compilation,
%                                requires installed pdfLaTeX, possible
%                                evalf danger, boolean [1 x 1]
%   'Compression'                turn on/off direct visibility algorithm,
%                                boolean [1 x 1]
%   'CoordinatesCross'           turn on/off coordinates system cross,
%                                boolean [1 x 1]
%   'FileName'                   character string with resulting file
%                                name
%   'OriginCoordSystem'          parameter  allows translation of origin of
%                                coordinates system, double [2 x 1]
%   'Standalone'                 sets file to be standalone or not, 
%                                boolean [1 x 1]
%   'TikZ_Corners'               value of corners rounding, double [1 x 1]
%   'TikZ_Opacity'               opacity of all triangles or particular ones
%                                double [1 x 1] or [N x 1]
%   'TikZ_Scale'                 picture scale, double [1 x 1]
%   'TikZ_Shading'               turn on/off mesh shading, double [1 x 1]
%
%  OUTPUTS (optional):
%   data                         data structure, see particular
%                                documentation for each function
%   setttings                    common settings used by plots
%
%  SYNTAX:
%   plot2TikZ(hndl,___)
%   plot2TikZ(hndl,'settings',settings,____)
%   plot2TikZ([],'settings',settings,'data',data)
%   data = plot2TikZ(hndl,____)
%   [data,settings] = plot2TikZ(hndl,____)
%
%  2020, Vojtech Neuman, neumavo1@fel.cvut.cz (2020)

p = inputParser;
p.CaseSensitive = false;

%% Default Settings
defaultData =                    [];
defaultSettings =                [];

defaultArrowThresh =             0.5;
defaultBoundingSphere =          false;
defaultColorbar =                false;
defaultCompileAutomatically =    true;
defaultCompression =             true;
defaultCoordinatesCross =        false;
defaultFileName =                'defaultFileName';

defaultTikZ_Corners =            0.1;
defaultTikZ_Opacity =            1;
defaultTikZ_Scale =              2;
defaultTikZ_Shading =            true;

%% Optional Arguments
p.addParameter('Data', defaultData,...
   @(Data) validateattributes(Data, {'struct'}, {'nonempty'}));
p.addParameter('Settings', defaultSettings,...
   @(Settings) validateattributes(Settings, {'struct'},{'nonempty'}));

p.addParameter('ArrowThresh', defaultArrowThresh,...
   @(ArrowThresh) validateattributes(ArrowThresh,...
   {'double'}, {'nonempty','size',[1 1],'>=',0,'<',1}));
p.addParameter('BoundingSphere', defaultBoundingSphere,...
   @(BoundingSphere) validateattributes(BoundingSphere,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('Colorbar', defaultColorbar,...
   @(Colorbar) validateattributes(Colorbar,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('CompileAutomatically', defaultCompileAutomatically,...
   @(CompileAutomatically) validateattributes(CompileAutomatically,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('Compression', defaultCompression,...
   @(Compression) validateattributes(Compression,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('CoordinatesCross', defaultCoordinatesCross,...
   @(CoordinatesCross) validateattributes(CoordinatesCross,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('FileName', defaultFileName,...
   @(FileName) validateattributes(FileName,...
   {'char'}, {'nonempty'}));

p.addParameter('TikZ_Corners', defaultTikZ_Corners,...
   @(TikZ_Corners) validateattributes(TikZ_Corners,...
   {'double'}, {'nonempty','size',[1 1]}));
p.addParameter('TikZ_Opacity', defaultTikZ_Opacity,...
   @(TikZ_Opacity) validateattributes(TikZ_Opacity,...
   {'double'}, {'nonempty','size',[NaN 1],'>',0,'<=',1}));
p.addParameter('TikZ_Scale', defaultTikZ_Scale,...
   @(TikZ_Scale) validateattributes(TikZ_Scale,...
   {'double'}, {'nonempty','size',[1 1]}));
p.addParameter('TikZ_Shading',defaultTikZ_Shading,...
   @(TikZ_Shading) validateattributes(TikZ_Shading,...
   {'logical'}, {'nonempty','size',[1 1]}));

if isvector(azimuth) && isvector(elevation)
   if isscalar(azimuth)
      azimuth = azimuth*ones(length(elevation),1);
   elseif isscalar(elevation)
      elevation = elevation*ones(length(azimuth),1);
   elseif length(azimuth) ~= length(elevation)
      error('Wrong size of angle vectors.');
   end
elseif isempty(azimuth) && isempty(elevation)
   error('Wrong size of angle vectors.');
end

%% Settings Parse
p.parse(varargin{:});
if isempty(p.Results.Settings)
   settings.ArrowThresh =           p.Results.ArrowThresh;
   settings.BoundingSphere =        p.Results.BoundingSphere;
   settings.Colorbar =              p.Results.Colorbar;
   settings.CompileAutomatically =  p.Results.CompileAutomatically;
   settings.Compression =           p.Results.Compression;
   settings.CoordinatesCross =      p.Results.CoordinatesCross;
   settings.FileName =              p.Results.FileName;
   settings.TikZ_Corners =          p.Results.TikZ_Corners;
   settings.TikZ_Opacity =          p.Results.TikZ_Opacity;
   settings.TikZ_Scale =            p.Results.TikZ_Scale;
else
   settings = p.Results.Settings;
   if ~isfield(settings,'FileName')
      settings.FileName = p.Results.FileName;
   end
end
settings.CircumSphere =       true;
settings.Colorbar =           false;
settings.OpenAfter =          false;
settings.OriginCoordSystem =  [0;0];
settings.Standalone =         true;
FileName =                    settings.FileName;

% creation of folder
mkdir([FileName 'Folder'])
currentDirectory = [pwd '\' FileName 'Folder'];

%% Execution of Animation Parts
for j = 1:length(azimuth)
   settings.FileName = [FileName 'Folder' '\' FileName num2str(j)];
   [~,~,radiusOfCircumSphere] = export.plot2TikZ(hndl,'azimuth',azimuth(j),...
      'elevation',elevation(j),'settings',settings);
   delete([FileName num2str(j) '.aux']);
   delete([FileName num2str(j) '.log']);
   delete([FileName num2str(j) '.synctex.gz']);
   movefile([FileName num2str(j) '.pdf'],currentDirectory)
end

%% Strings Preparation
hwSize = sqrt(1.9*radiusOfCircumSphere^2*pi*32.4)*settings.TikZ_Scale/5;
animateStr = sprintf([...
   '\\documentclass[tikz]{beamer}\n'...
   '\\usepackage{animate}\n'...
   '\n'...
   '%% Beamer set-up\n'...
   '\\beamertemplatenavigationsymbolsempty\n'...
   '\\usepackage[orientation=landscape,size=custom,width=%1.2f,height=%1.2f,debug]'...
   '{beamerposter}\n'...
   '\\setbeamersize{text margin left=0em}\n'...
   '\\setbeamersize{text margin right=0em}\n'...
   '\n'...
   '\\begin{document}\n'...
   '	\\begin{center}\n'...
   '		\\begin{frame}\n'...
   '			\\animategraphics[loop, autoplay, controls,buttonsize=%1.3fem,]'...
   '{%1.0f}{%s}{1}{%1.0f}\n'...
   '		\\end{frame}\n'...
   '	\\end{center}\n'...
   '\\end{document}\n'],hwSize,1.02*hwSize,hwSize/20,...
   floor(length(azimuth)/4),FileName,length(azimuth));

%% Strings Printing
fid = fopen([FileName '.tex'], 'w+');
fprintf(fid,'%s',animateStr);
fclose(fid);
end