function varargout = plot2TikZ(hndl,varargin)
%% plot2TikZ() serves as interface between MATLAB and LaTeX
% Converts AToM plots into TikZ macros.
%
%  INPUTS:
%   hndl                         structure with particular plot handles
%
%  INPUTS (optional):
%   'Data'                       data structure containing main plot
%                                information
%   'Settings'                   settings structure containing
%                                all or partial parameters
%   'Azimuth'                    specified azimuthal angle (have to be
%                                set together with 'elevation'),
%                                double [1 x 1]
%   'Elevation'                  specified elevational angle (have to be
%                                set together with 'azimuth'),
%                                double [1 x 1]
%   'Correction'                 factor for possible size correction,
%                                
%  Settable parts of Settings structure
%   'ArrowThresh'                sets threshold for omitting arrows,
%                                double [1 x 1]
%   'BoundingSphere'             draws gray circumscribing sphere around
%                                object, boolean [1 x 1]
%   'CircumSphere'               draws white sphere around whole picture,
%                                prepares data for animation, boolean [1 x 1]
%   'Colorbar'                   turn on/off colorbar with colormap
%                                boolean [1 x 1]
%   'CompileAutomatically'       turn on/off automatic compilation,
%                                requires installed pdfLaTeX, possible
%                                evalf danger, boolean [1 x 1]
%   'Compression'                turn on/off direct visibility algorithm,
%                                boolean [1 x 1]
%   'CoordinatesCross'           turn on/off coordinates system cross,
%                                boolean [1 x 1]
%   'FileName'                   character string with resulting file
%                                name
%   'OriginCoordSystem'          parameter  allows translation of origin of
%                                coordinates system, double [2 x 1]
%   'Standalone'                 sets file to be standalone or not, 
%                                boolean [1 x 1]
%   'TikZ_Corners'               value of corners rounding, double [1 x 1]
%   'TikZ_Opacity'               opacity of all triangles or particular ones
%                                double [1 x 1] or [N x 1]
%   'TikZ_Scale'                 picture scale, double [1 x 1]
%   'TikZ_Shading'               turn on/off mesh shading, double [1 x 1]
%
%  OUTPUTS (optional):
%   data                         data structure, see particular
%                                documentation for each function
%   setttings                    common settings used by plots
%
%  SYNTAX:
%   plot2TikZ(hndl,___)
%   plot2TikZ(hndl,'settings',settings,____)
%   plot2TikZ([],'settings',settings,'data',data)
%   data = plot2TikZ(hndl,____)
%   [data,settings] = plot2TikZ(hndl,____)
%
%  2020, Vojtech Neuman, neumavo1@fel.cvut.cz (2020)

%% Optional settings
p = inputParser;
p.CaseSensitive = false;

% default settings
defaultAzimuth =                 [];
defaultCorrection =              1;
defaultData =                    [];
defaultElevation =               [];
defaultSettings =                [];

defaultArrowThresh =             0.1;
defaultBoundingSphere =          false;
defaultCircumSphere =            false;
defaultColorbar =                false;
defaultCompileAutomatically =    true;
defaultCompression =             true;
defaultCoordinatesCross =        false;
defaultFileName =                ['AToM2TiKZ' num2str(now)];
defaultOriginCoordSystem =       [0;0];
defaultStandalone =              true;

defaultTikZ_Corners =            0.1;
defaultTikZ_Opacity =            1;
defaultTikZ_Scale =              5;
defaultTikZ_Shading =            false;

% optional arguments
p.addParameter('Azimuth', defaultAzimuth,...
   @(Azimuth) validateattributes(Azimuth,...
   {'double'}, {'nonempty','size',[1 1]}));
p.addParameter('Correction', defaultCorrection,...
   @(Correction) validateattributes(Correction,...
   {'double'}, {'nonempty','size',[1 1],'>',0}));
p.addParameter('Elevation', defaultElevation,...
   @(Elevation) validateattributes(Elevation,...
   {'double'}, {'nonempty','size',[1 1]}));
p.addParameter('Data', defaultData,...
   @(Data) validateattributes(Data,...
   {'struct'}, {'nonempty'}));
p.addParameter('Settings', defaultSettings,...
   @(Settings) validateattributes(Settings,...
   {'struct'},{'nonempty'}));

p.addParameter('ArrowThresh', defaultArrowThresh,...
   @(ArrowThresh) validateattributes(ArrowThresh,...
   {'double'}, {'nonempty','size',[1 1],'>=',0,'<',1}));
p.addParameter('BoundingSphere', defaultBoundingSphere,...
   @(BoundingSphere) validateattributes(BoundingSphere,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('CircumSphere', defaultCircumSphere,...
   @(CircumSphere) validateattributes(CircumSphere,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('Colorbar', defaultColorbar,...
   @(Colorbar) validateattributes(Colorbar,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('CompileAutomatically', defaultCompileAutomatically,...
   @(CompileAutomatically) validateattributes(CompileAutomatically,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('Compression', defaultCompression,...
   @(Compression) validateattributes(Compression,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('CoordinatesCross', defaultCoordinatesCross,...
   @(CoordinatesCross) validateattributes(CoordinatesCross,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('FileName', defaultFileName,...
   @(FileName) validateattributes(FileName,...
   {'char'}, {'nonempty'}));
p.addParameter('OriginCoordSystem', defaultOriginCoordSystem,...
   @(OriginCoordSystem) validateattributes(OriginCoordSystem,...
   {'double'}, {'nonempty','size',[2 1]}));
p.addParameter('Standalone', defaultStandalone,...
   @(Standalone) validateattributes(Standalone,...
   {'logical'}, {'nonempty','size',[1 1]}));

p.addParameter('TikZ_Corners', defaultTikZ_Corners,...
   @(TikZ_Corners) validateattributes(TikZ_Corners,...
   {'double'}, {'nonempty','size',[1 1]}));
p.addParameter('TikZ_Opacity', defaultTikZ_Opacity,...
   @(TikZ_Opacity) validateattributes(TikZ_Opacity,...
   {'double'}, {'nonempty','size',[NaN 1],'>',0,'<=',1}));
p.addParameter('TikZ_Scale', defaultTikZ_Scale,...
   @(TikZ_Scale) validateattributes(TikZ_Scale,...
   {'double'}, {'nonempty','size',[1 1]}));
p.addParameter('TikZ_Shading',defaultTikZ_Shading,...
   @(TikZ_Shading) validateattributes(TikZ_Shading,...
   {'logical'}, {'nonempty','size',[1 1]}));

% settings and data acquisition
p.parse(varargin{:});
data = p.Results.Data;
corrVert = p.Results.Correction;
settings = settingsParse(p.Results.Settings);


%% Transformation matrix and settings
if ~isempty(hndl) && isempty(settings)
   if isempty([p.Results.Azimuth p.Results.Elevation])
      VW = hndl.axes.View;
      u = VW(1)*pi/180;
      w = VW(2)*pi/180;
      Rz = [cos(u) -sin(u) 0; sin(u) cos(u) 0; 0 0 1];
      Rx = [1 0 0; 0 cos(w) sin(w); 0 -sin(w) cos(w)];
   elseif ~isempty(p.Results.Elevation) && ~isempty(p.Results.Azimuth)
      hndl.axes.View = [p.Results.Azimuth p.Results.Elevation];
      VW = hndl.axes.View;
      u = VW(1)*pi/180;
      w = VW(2)*pi/180;
      Rz = [cos(u) -sin(u) 0; sin(u) cos(u) 0; 0 0 1];
      Rx = [1 0 0; 0 cos(w) sin(w); 0 -sin(w) cos(w)];
   else
      error('Insufficient angle settings.');
   end
   
   settings.ArrowThresh =           p.Results.ArrowThresh;
   settings.BoundingSphere =        p.Results.BoundingSphere;
   settings.CircumSphere =          p.Results.CircumSphere;
   settings.Colorbar =              p.Results.Colorbar;
   settings.CompileAutomatically =  p.Results.CompileAutomatically;
   settings.Compression =           p.Results.Compression;
   settings.CoordinatesCross =      p.Results.CoordinatesCross;
   settings.FileName =              p.Results.FileName;
   settings.OpenAfter =             true;
   settings.OriginCoordSystem =     p.Results.OriginCoordSystem;
   settings.Standalone =            p.Results.Standalone;
   settings.TransMat3D =            Rz*Rx*[1 0 0; 0 0 1; 0 -1 0];
   settings.VectorsCoordSystem =    eye(3);
   settings.TikZ_Corners =          p.Results.TikZ_Corners;
   settings.TikZ_Opacity =          p.Results.TikZ_Opacity;
   settings.TikZ_Scale =            p.Results.TikZ_Scale;
   settings.TikZ_Shading =          p.Results.TikZ_Shading;
   data.type =                      hndl.type;
   
elseif ~isempty(hndl) && ~isempty(settings)
   if isempty([p.Results.Azimuth p.Results.Elevation])
      if isempty(settings.TransMat3D)
         VW = hndl.axes.View;
         u = VW(1)*pi/180;
         w = VW(2)*pi/180;
         Rz = [cos(u) -sin(u) 0; sin(u) cos(u) 0; 0 0 1];
         Rx = [1 0 0; 0 cos(w) sin(w); 0 -sin(w) cos(w)];
         settings.TransMat3D = Rz*Rx*[1 0 0; 0 0 1; 0 -1 0];
      end
   elseif ~isempty(p.Results.Elevation) && ~isempty(p.Results.Azimuth)
      hndl.axes.View = [p.Results.Azimuth p.Results.Elevation];
      VW = hndl.axes.View;
      u = VW(1)*pi/180;
      w = VW(2)*pi/180;
      Rz = [cos(u) -sin(u) 0; sin(u) cos(u) 0; 0 0 1];
      Rx = [1 0 0; 0 cos(w) sin(w); 0 -sin(w) cos(w)];
      settings.TransMat3D = Rz*Rx*[1 0 0; 0 0 1; 0 -1 0];
   else
      error('Insufficient angle settings.');
   end
   data.type = hndl.type;
elseif ~isempty(data) && ~isempty(settings) && isempty(hndl)
   if isempty([p.Results.Azimuth p.Results.Elevation])
      if isempty(settings.TransMat3D)
         settings.TransMat3D = [1 0 0; 0 0 1; 0 -1 0];
      end
   elseif ~isempty(p.Results.Elevation) && ~isempty(p.Results.Azimuth)
      u = p.Results.Azimuth*pi/180;
      w = p.Results.Elevation*pi/180;
      Rz = [cos(u) -sin(u) 0; sin(u) cos(u) 0; 0 0 1];
      Rx = [1 0 0; 0 cos(w) sin(w); 0 -sin(w) cos(w)];
      settings.TransMat3D = Rz*Rx*[1 0 0; 0 0 1; 0 -1 0];
   else
      error('Insufficient angle settings.');
   end
elseif isempty(hndl) && ~isempty(data)
   if isempty([p.Results.Azimuth p.Results.Elevation])
      u = 0;
      w = pi/2;
      Rz = [cos(u) -sin(u) 0; sin(u) cos(u) 0; 0 0 1];
      Rx = [1 0 0; 0 cos(w) sin(w); 0 -sin(w) cos(w)];
   elseif ~isempty(p.Results.Elevation) && ~isempty(p.Results.Azimuth)
      u = p.Results.Azimuth*pi/180;
      w = p.Results.Elevation*pi/180;
      Rz = [cos(u) -sin(u) 0; sin(u) cos(u) 0; 0 0 1];
      Rx = [1 0 0; 0 cos(w) sin(w); 0 -sin(w) cos(w)];
   else
      error('Insufficient angle settings.');
   end
   
   settings.ArrowThresh =           p.Results.ArrowThresh;
   settings.BoundingSphere =        p.Results.BoundingSphere;
   settings.CircumSphere =          p.Results.CircumSphere;
   settings.Colorbar =              p.Results.Colorbar;
   settings.CompileAutomatically =  p.Results.CompileAutomatically;
   settings.Compression =           p.Results.Compression;
   settings.CoordinatesCross =      p.Results.CoordinatesCross;
   settings.FileName =              p.Results.FileName;
   settings.OpenAfter =             true;
   settings.OriginCoordSystem =     [0;0;0];
   settings.Standalone =            p.Results.Standalone;
   settings.TransMat3D =            Rz*Rx*[1 0 0; 0 0 1; 0 -1 0];
   settings.VectorsCoordSystem =    eye(3);
   settings.TikZ_Corners =          p.Results.TikZ_Corners;
   settings.TikZ_Opacity =          p.Results.TikZ_Opacity;
   settings.TikZ_Scale =            p.Results.TikZ_Scale;
   settings.TikZ_Shading =          p.Results.TikZ_Shading;
else
   error('Handles and data cannot be set both at the same time.')
end

%% Export Functions
if ~isempty(hndl)
   if strcmp(data.type,'charge')
      data.faces_b =      hndl.triangles.Faces;
      data.vert =         corrVert*hndl.triangles.Vertices;
      data.cMap =         hndl.axes.Colormap;
      cVec =              hndl.triangles.CData;
      colMat =            [hndl.axes.CLim(2) 1; hndl.axes.CLim(1) 1];
      colV =              colMat\[size(data.cMap,1);1];
      data.cVec =         round(colV(1)*cVec+colV(2));
      
      radiusOfCircumSphere = export.advanced.charge2TikZ(data,settings);
   elseif strcmp(data.type,'current')
      data.faces_b =       hndl.triangles.Faces;
      data.vert =          corrVert*hndl.triangles.Vertices;
      data.cMap =          hndl.axes.Colormap;
      cVec =               hndl.triangles.CData;
      colMat =             [hndl.axes.CLim(2) 1; hndl.axes.CLim(1) 1];
      colV =               colMat\[size(data.cMap,1);1];
      data.cVec =          round(colV(1)*cVec+colV(2));
      data.arrowsVert =    corrVert*hndl.arrow(2).Vertices;
      data.arrows =        corrVert*[...
         hndl.arrow(3).XData(2,:)'...
         hndl.arrow(3).YData(2,:)'...
         hndl.arrow(3).ZData(2,:)'...
         hndl.arrow(3).XData(3,:)'...
         hndl.arrow(3).YData(3,:)'...
         hndl.arrow(3).ZData(3,:)'];
      
      radiusOfCircumSphere = export.advanced.current2TikZ(data,settings);
   elseif strcmp(data.type,'farField')
      data.faces_b =      hndl.farFieldSurface.Faces;
      data.vert =         corrVert*hndl.farFieldSurface.Vertices;
      data.cMap =         hndl.axes.Colormap;
      cVec =              (mean(hndl.farFieldSurface.CData))';
      colMat =            [hndl.axes.CLim(2) 1; hndl.axes.CLim(1) 1];
      colV =              colMat\[size(data.cMap,1);1];
      data.cVec =         round(colV(1)*cVec+colV(2));
      
      radiusOfCircumSphere = export.advanced.farField2TikZ(data,settings);
   elseif strcmp(data.type,'mesh') && isfield(hndl,'triangles')
      data.faces_b = hndl.triangles.Faces;
      data.vert = corrVert*hndl.triangles.Vertices;
      data.cVec = hndl.triangles.FaceVertexCData;
      
      radiusOfCircumSphere = export.advanced.mesh2TikZ(data,settings);
   elseif strcmp(data.type,'topoSens')
      data.Mesh = hndl.Mesh;
      data.BF = hndl.BF;
      data.Temp = hndl.Temp(:,1);
      
      radiusOfCircumSphere = export.advanced.topoSens2TikZ(data,settings);
   end
elseif ~isempty(data)
   if strcmp(data.type,'charge')
      temp = [...
         isfield(data,'faces_b')...
         isfield(data,'vert')...
         isfield(data,'cMap')...
         isfield(data,'cVec')];
      
      if sum(temp) == 4
         radiusOfCircumSphere = export.advanced.charge2TikZ(data,settings);
      else
         error('Data structure is not complete');
      end
      
   elseif strcmp(data.type,'current')
      temp = [...
         isfield(data,'faces_b')...
         isfield(data,'vert')...
         isfield(data,'cMap')...
         isfield(data,'cVec')...
         isfield(data,'arrows')];
      
      if sum(temp) == 5
         radiusOfCircumSphere = export.advanced.current2TikZ(data,settings);
      else
         error('Data structure is not complete');
      end
      
   elseif strcmp(data.type,'farField')
      temp = [...
         isfield(data,'faces_b')...
         isfield(data,'vert')...
         isfield(data,'cMap')...
         isfield(data,'cVec')];
      
      if sum(temp) == 4
         radiusOfCircumSphere = export.advanced.farField2TikZ(data,settings);
      else
         error('Data structure is not complete');
      end
      
   elseif strcmp(data.type,'mesh')
      temp = [...
         isfield(data,'faces_b')...
         isfield(data,'vert')...
         isfield(data,'cVec')];
      
      if sum(temp) == 3
         radiusOfCircumSphere = export.advanced.mesh2TikZ(data,settings);
      end
   else
      error('Wrong or not supported type')
   end
end

%% Output arguments handling
if nargout == 1
   varargout{1} = data;
elseif nargout == 2
   varargout{1} = data;
   varargout{2} = settings;
elseif nargout == 3
   varargout{1} = data;
   varargout{2} = settings;
   varargout{3} = radiusOfCircumSphere;
end
end

function settings = settingsParse(settings)
%% Function match user settings with missing properties.

%  check if settings is empty structure
if isempty(settings)
   settings = [];
   return
end

%  else parse input
p = inputParser;
p.CaseSensitive = false;

% default settings
defaultArrowThresh =             0.5;
defaultBoundingSphere =          false;
defaultCircumSphere =            false;
defaultColorbar =                false;
defaultCompileAutomatically =    false;
defaultCompression =             true;
defaultCoordinatesCross =        false;
defaultFileName =                ['AToM2TiKZ' num2str(now)];
defaultOpenAfter =               true;
defaultOriginCoordSystem =       [0; 0];
defaultStandalone =              true;
defaultTransMat3D =              [];
defaultVectorsCoordSystem =      eye(3);

defaultTikZ_Corners =            0.1;
defaultTikZ_Opacity =            1;
defaultTikZ_Scale =              2;
defaultTikZ_Shading =            true;

% ouput file specifications
p.addParameter('ArrowThresh', defaultArrowThresh,...
   @(ArrowThresh) validateattributes(ArrowThresh,...
   {'double'}, {'nonempty','size',[1 1],'>=',0,'<',1}));
p.addParameter('BoundingSphere', defaultBoundingSphere,...
   @(BoundingSphere) validateattributes(BoundingSphere,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('CircumSphere', defaultCircumSphere,...
   @(CircumSphere) validateattributes(CircumSphere,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('Colorbar', defaultColorbar,...
   @(Colorbar) validateattributes(Colorbar,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('CompileAutomatically', defaultCompileAutomatically,...
   @(CompileAutomatically) validateattributes(CompileAutomatically,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('Compression', defaultCompression,...
   @(Compression) validateattributes(Compression,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('CoordinatesCross', defaultCoordinatesCross,...
   @(CoordinatesCross) validateattributes(CoordinatesCross,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('FileName', defaultFileName,...
   @(FileName) validateattributes(FileName,...
   {'char'}, {'nonempty'}));
p.addParameter('OpenAfter', defaultOpenAfter,...
   @(OpenAfter) validateattributes(OpenAfter,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('OriginCoordSystem', defaultOriginCoordSystem,...
   @(OriginCoordSystem) validateattributes(OriginCoordSystem,...
   {'double'}, {'nonempty','size',[2 1]}));
p.addParameter('Standalone', defaultStandalone,...
   @(Standalone) validateattributes(Standalone,...
   {'logical'}, {'nonempty','size',[1 1]}));
p.addParameter('TransMat3D', defaultTransMat3D,...
   @(TransMat3D) validateattributes(TransMat3D,...
   {'double'}, {'nonempty','size',[3 3]}));
p.addParameter('VectorsCoordSystem', defaultVectorsCoordSystem,...
   @(VectorsCoordSystem) validateattributes(VectorsCoordSystem,...
   {'double'}, {'nonempty','size',[3 3]}));

% TikZ settingss
p.addParameter('TikZ_Corners', defaultTikZ_Corners,...
   @(TikZ_Corners) validateattributes(TikZ_Corners,...
   {'double'}, {'nonempty','size',[1 1]}));
p.addParameter('TikZ_Opacity', defaultTikZ_Opacity,...
   @(TikZ_Opacity) validateattributes(TikZ_Opacity,...
   {'double'}, {'nonempty','size',[NaN 1],'>',0,'<=',1}));
p.addParameter('TikZ_Scale', defaultTikZ_Scale,...
   @(TikZ_Scale) validateattributes(TikZ_Scale,...
   {'double'}, {'nonempty','size',[1 1]}));
p.addParameter('TikZ_Shading',defaultTikZ_Shading,...
   @(TikZ_Shading) validateattributes(TikZ_Shading,...
   {'logical'}, {'nonemptu','size',[1 1]}));

% parsing output
settingsNames = fieldnames(settings);
settingsValues = struct2cell(settings);
settings = {[settingsNames settingsValues]};
settings = reshape(settings{:}',[numel(settings{:}) 1]);
p.parse(settings{:});
settings = p.Results;
end