%% TikZ Conversion Functions Example
% Example of usage of plot conversion to TikZ macros.
%
% AToM2TikZ expansion, beta version
%  2020, Vojtech Neuman, neumavo1@fel.cvut.cz (2020)
clc;
clear;
close all;

% Setting of project name and launch of AToM.
fileName = 'TikZ_Example';
fileName = models.utilities.prepareExampleSession(fileName, 'atom');
atom = Atom.start(false);
atom.createProject(fileName);

% Creation of variables.
atom.selectedProject.workspace.addVariable('L', 0.6, 'half length');
atom.selectedProject.workspace.addVariable('W', 0.5, 'half width');

% Particular frequencies of interest.
nFrequencies = 5;
frequencyList = linspace(100e6, 500e6, nFrequencies);
atom.selectedProject.physics.setFrequencyList(frequencyList);

% Preparation of geometrical model.
atom.selectedProject.geom.addParallelogram('[-L/2, -W/2, 0]', ...
   '[L/2, -W/2, 0]', '[-L/2, W/2, 0]', 'dipole1');
atom.selectedProject.geom.addParallelogram('[-L/2, -W/2, 0.1]', ...
   '[L/2, -W/2, 0.1]', '[-L/2, W/2, 0.1]', 'dipole2');

% Feeding specification.
atom.selectedProject.physics.feeding.add2DDiscretePort(1, ...
   'baseObjName', 'dipole1', ...
   'positionPar', '[0, -W/2, 0; 0, W/2, 0]', ...
   'description', 'port1');
atom.selectedProject.physics.feeding.add2DDiscretePort(2, ...
   'baseObjName', 'dipole2', ...
   'positionPar', '[-L/4, -W/2, 0.1; -L/4, W/2, 0.1]', ...
   'description', 'port2');
atom.selectedProject.physics.feeding.editFeeder(1, 'isEnabled', true, ...
   'type', 'voltage', 'value', '1');
atom.selectedProject.physics.feeding.editFeeder(2, 'isEnabled', true, ...
   'type', 'voltage', 'value', '1');

% Preparation of solver and its execution.
meshSize = 16;
atom.selectedProject.mesh.setGlobalMeshDensity( meshSize );
atom.selectedProject.mesh.getMesh();
atom.selectedProject.solver.MoM2D.setProperties('resultRequests', ...
   'basisFcns, mesh, iVec', ...
   'verbosity', 2, 'nBatchMax', 2000, ...
   'nWorkers', 0);
atom.selectedProject.solver.MoM2D.setProperties('quadOrder', 1);
atom.selectedProject.solver.MoM2D.solve();

% Acquisition of particular results structures.
mesh = atom.selectedProject.solver.MoM2D.results.mesh;
basisFcns = atom.selectedProject.solver.MoM2D.results.basisFcns;
iVec = atom.selectedProject.solver.MoM2D.results.iVec.data;

% Termination of AToM and command window cleaning.
atom.quit;
close all;
clc

% Plotting of results for the second frequency from the list.
hndl = results.plotCharge(mesh,...
   'basisFcns', basisFcns,...
   'iVec', iVec(:,1));
hndl.type = 'charge';

%% Simplest Way
% Direct use of plot2TikZ(), 'FileName' parameter is used for highlighting of
% resulting file. Otherwise, without specifying, 'defaultFileName' is used
% instead.
export.plot2TikZ(hndl,...
   'FileName','charge_density_3D');

%% Angles Setting
% Different view angle settings.
export.plot2TikZ(hndl,...
   'Azimuth',60,...
   'Elevation',30,...
   'FileName','charge_density_3D_angle',...
   'Colorbar',true);

%% Another Way of Opacity Settings
% Function allows set specific values for opacity. In this example is this
% parameter proportional to absolute value of charge density on particular
% triangle.
opacity = hndl.triangles.FaceVertexCData;
opacity = sqrt(abs(opacity)/max(abs(opacity)));
export.plot2TikZ(hndl,...
   'FileName','charge_density_3D_opacity',...
   'TikZ_Opacity',opacity);

%% Motion Pictures
% Animation with fixed elevational angle. Although parameter 'ArrowThresh' is 
% set, it has no effect on this particular type of plot. 'BoundingSphere'
% encloses geometrical object into circumscribed sphere.
azimuth = 0:15:345;
elevation = 40;
export.animatePlot2TikZ(hndl,azimuth,elevation,...
   'FileName','charge_density_3D_animate',...
   'ArrowThresh',0.1,...
   'CompileAutomatically',true,...,
   'CoordinatesCross',true,...
   'Compression',true,...
   'TikZ_Scale',5)