%% TikZ Conversion Functions Example
% Example of usage of plot conversion to TikZ macros.
%
% AToM2TikZ expansion, beta version
%  2020, Vojtech Neuman, neumavo1@fel.cvut.cz (2020)
clc;
clear;
close all;

% Setting of project name and launch of AToM.
fileName = 'TikZ_Example';
fileName = models.utilities.prepareExampleSession(fileName, 'atom');
atom = Atom.start(false);
atom.createProject(fileName);

% Creation of variables.
atom.selectedProject.workspace.addVariable('L', 0.5, 'half length');
atom.selectedProject.workspace.addVariable('W', 0.6, 'half width');

% Particular frequencies of interest.
nFrequencies = 5;
frequencyList = linspace(100e6, 500e6, nFrequencies);
atom.selectedProject.physics.setFrequencyList(frequencyList);

% Preparation of geometrical model.
atom.selectedProject.geom.addParallelogram('[-L/2, -W/2, 0]', ...
   '[L/2, -W/2, 0]', '[-L/2, W/2, 0]', 'dipole1');
atom.selectedProject.geom.addParallelogram('[-L/2, -W/2, 0.1]', ...
   '[L/2, -W/2, 0.1]', '[-L/2, W/2, 0.1]', 'dipole2');

% Feeding specification.
atom.selectedProject.physics.feeding.add2DDiscretePort(1, ...
   'baseObjName', 'dipole1', ...
   'positionPar', '[0, -W/2, 0; 0, W/2, 0]', ...
   'description', 'port1');
atom.selectedProject.physics.feeding.add2DDiscretePort(2, ...
   'baseObjName', 'dipole2', ...
   'positionPar', '[-L/4, -W/2, 0.1; -L/4, W/2, 0.1]', ...
   'description', 'port2');
atom.selectedProject.physics.feeding.editFeeder(1, 'isEnabled', true, ...
   'type', 'voltage', 'value', '-1');
atom.selectedProject.physics.feeding.editFeeder(2, 'isEnabled', true, ...
   'type', 'voltage', 'value', '-2');

% Preparation of solver and its execution.
meshSize = 16;
atom.selectedProject.mesh.setGlobalMeshDensity( meshSize );
atom.selectedProject.mesh.getMesh();
atom.selectedProject.solver.MoM2D.setProperties('resultRequests', ...
   'basisFcns, mesh, iVec', ...
   'verbosity', 2, 'nBatchMax', 2000, ...
   'nWorkers', 0);
atom.selectedProject.solver.MoM2D.setProperties('quadOrder', 1);
atom.selectedProject.solver.MoM2D.solve();

% Acquisition of particular results structures.
mesh = atom.selectedProject.solver.MoM2D.results.mesh;
basisFcns = atom.selectedProject.solver.MoM2D.results.basisFcns;
iVec = atom.selectedProject.solver.MoM2D.results.iVec.data;

% Termination of AToM and command window cleaning.
atom.quit;
close all;
clc

% Plotting of results for the second frequency from the list.
hndl = results.plotCurrent(mesh,...
   'basisFcns',basisFcns,...
   'iVec', iVec(:,2),...
   'arrowScale', 'proportional');
hndl.type = 'current';

%% Simplest Way
% Everything is set by default. Resulting file name is 'AToM2TikZ_now().tex'.
% Nevertheless, it is changed there for better orientation to
% 'current_density_3D.tex'
export.plot2TikZ(hndl,...
   'FileName','current_density_3D');

%% Angle and FileName Settings
% Angle specification is independent of settings structure. This example
% represents choice view(70,40), units are degrees. This angle is then set back
% to figure handle for quick comparison of resulting .pdf file to former plot.
export.plot2TikZ(hndl,...
   'Azimuth',70,...
   'Elevation',40,...
   'FileName','current_density_3D_angle',...
   'CoordinatesCross',true);

%% Output Files
% In case of specified output, plot2TikZ() returns data or both data and
% settings for possible reuse without new execution of former script.
% 'CircumSphere' parameter adds white surrounding space which ensures smooth
% animation. For normal use it should stay turned off. 'CoordinatesCross' adds
% defautly coordinates cross into left bottom corner. This is changed with
% parameter to (0.75,0.75) coordinate .
[data,settings] = export.plot2TikZ(hndl,...
   'FileName','current_density_3D_axes',...
   'CoordinatesCross',true,...
   'CircumSphere',true,...
   'OriginCoordSystem',[0.75; 0.75],...
   'Compression',false,...
   'CompileAutomatically',true,...
   'TikZ_Scale',5);

%% Used Presaved Data
% This example show how to use already prepared data and settings to create
% similar plot with slight changes. Compare size of previous document with this
% one for observation of 'Compression' effect.
settings.FileName = 'current_density_3D_compression';
settings.Compression = true;
export.plot2TikZ([],...
   'Data',data,...
   'Settings',settings);

%% Whole settings structure.
% Data and settings are cleared for safety.
clear settings data

% Example of setting all possible features suported by AToM2TikZ extension.
% 'ArrowThresh' parameter allows to omit too small current arrows. In this
% example is also utilized 'Colorbar', which draws colorbar in right side.
% 'BoundingSphere' draws gray circumscribed sphere around object. Afterwards
% automatic compilation is potentially dangerous. Evaluation can end in
% intracable error due to use of command prompt scripts. 'CompileAutomatically'
% is prepared only for compilation on Microsoft Windows . 
settings.ArrowThresh = 0.1;
settings.BoundingSphere = true;
settings.CircumSphere = true;
settings.Colorbar = true;
settings.CompileAutomatically = true;
settings.Compression = true;
settings.CoordinatesCross = true;
settings.FileName = 'current_density_3D_settings';
settings.Standalone = true;
settings.TikZ_Scale = 6;
settings.TikZ_Opacity = 1;
export.plot2TikZ(hndl,...
   'settings',settings);

%% Opacity
% Opacity is at default set to 1. It is possible to change to arbitraty
% scalar between [0 1]. However, it can also be set with help of user defined
% specification. In this example is chosen partial visibility of upper desk.
% The parameter is restricted only to be a scalar or a vector of same size as
% number of triangles in mesh and to user imagination.
opacity = ones(mesh.nTriangles,1);
opacity(mesh.triangleCentroids(:,3) > 0.05) = 0.15;
export.plot2TikZ(hndl,...
   'fileName','current_density_3D_opacity',...
   'TikZ_Opacity',opacity);

%% Motion Pictures
% Animation has by default set 'CircumSphere' to true without possibility to
% change it. Length of the angle vectors determines resulting animation. For
% clarity, all files are set in folder with similar name to specified
% 'fileName'.
azimuth = 0:12:348;
elevation = [-90:12:90 78:-12:-78];
export.animatePlot2TikZ(hndl,azimuth,elevation,...
   'FileName','current_density_3D_animate',...
   'ArrowThresh',0.25,...
   'BoundingSphere',true)

% Problem with turn off auto compilation, directory tries move file that DNE